@@ -1,11 +1,13 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
 /* GNOME panel mail check module.
  * (C) 1997, 1998, 1999, 2000 The Free Software Foundation
+ * (C) 2001 Eazel, Inc.
  *
  * Authors: Miguel de Icaza
  *          Jacob Berkman
  *          Jaka Mocnik
  *          Lennart Poettering
+ *          George Lebl
  *
  */
 
@@ -22,6 +24,7 @@
 #include <gdk-pixbuf/gdk-pixbuf.h>
 
 #include "popcheck.h"
+#include "remote-helper.h"
 #include "mailcheck.h"
 
 GtkWidget *applet = NULL;
@@ -122,10 +125,11 @@ struct _MailCheck {
 	GtkWidget *mailfile_entry, *mailfile_label, *mailfile_fentry;
 	GtkWidget *remote_server_entry, *remote_username_entry, *remote_password_entry;
 	GtkWidget *remote_server_label, *remote_username_label, *remote_password_label;
+	GtkWidget *pre_remote_command_label, *pre_remote_command_entry;
 	GtkWidget *remote_option_menu;
 	GtkWidget *play_sound_check;
         
-	char *remote_server, *remote_username, *remote_password, *real_password;
+	char *pre_remote_command, *remote_server, *remote_username, *remote_password, *real_password;
 	MailboxType mailbox_type; /* local = 0; maildir = 1; pop3 = 2; imap = 3 */
         MailboxType mailbox_type_temp;
 
@@ -134,9 +138,13 @@ struct _MailCheck {
 	int type; /*mailcheck = 0; mailbox = 1 */
 	
 	int size;
+
+	/* see remote-helper.h */
+	gpointer remote_handle;
 };
 
 static int mail_check_timeout (gpointer data);
+static void after_mail_check (MailCheck *mc);
 
 #define WANT_BITMAPS(x) (x == REPORT_MAIL_USE_ANIMATION || x == REPORT_MAIL_USE_BITMAP)
 
@@ -204,6 +212,51 @@ get_remote_password (void)
 	return pass;
 }
 
+
+static void
+got_remote_answer (int mails, gpointer data)
+{
+	MailCheck *mc = data;
+	int old_unreadmail;
+
+	mc->remote_handle = NULL;
+	
+	if (mails == -1) {
+#if 0
+		/* don't notify about an error: think of people with
+		 * dial-up connections; keep the current mail status
+		 */
+		GtkWidget *box = NULL;
+		box = gnome_message_box_new (_("Remote-client-error occured. Remote-polling deactivated. Maybe you used a wrong server/username/password?"),
+					     GNOME_MESSAGE_BOX_ERROR, GNOME_STOCK_BUTTON_CLOSE, NULL);
+		gtk_window_set_modal (GTK_WINDOW(box),TRUE);
+		gtk_widget_show (box);
+		
+		mc->mailbox_type = MAILBOX_LOCAL;
+		mc->anymail = mc->newmail = 0;
+#endif
+	} else {
+		old_unreadmail = mc->unreadmail;
+		mc->unreadmail = (signed int) (((unsigned int) mails) >> 16);
+		if(mc->unreadmail > old_unreadmail) /* lt */
+			mc->newmail = 1;
+		else
+			mc->newmail = 0;
+		mc->totalmail = (signed int) (((unsigned int) mails) & 0x0000FFFFL);
+		mc->anymail = mc->totalmail ? 1 : 0;
+
+		after_mail_check (mc);
+	} 
+}
+
+static void
+null_remote_handle (gpointer data)
+{
+	MailCheck *mc = data;
+
+	mc->remote_handle = NULL;
+}
+
 /*
  * Get file modification time, based upon the code
  * of Byron C. Darrah for coolmail and reused on fvwm95
@@ -212,13 +265,15 @@ static void
 check_mail_file_status (MailCheck *mc)
 {
 	static off_t oldsize = 0;
-	struct stat s;
 	off_t newsize;
-	int status, old_unreadmail;
+	struct stat s;
+	int status;
 	
 	if ((mc->mailbox_type == MAILBOX_POP3) || 
 	    (mc->mailbox_type == MAILBOX_IMAP)) {
-		int v;
+		if (mc->remote_handle != NULL)
+			/* check in progress */
+			return;
 
 		if (mc->remote_password != NULL &&
 		    mc->remote_password[0] != '\0') {
@@ -235,44 +290,27 @@ check_mail_file_status (MailCheck *mc)
 							   mc);
 		}
 
-		if (mc->real_password && mc->remote_username && mc->remote_server) {
+		if (mc->real_password != NULL &&
+		    mc->remote_username != NULL &&
+		    mc->remote_server != NULL) {
 			if (mc->mailbox_type == MAILBOX_POP3)
-				v = pop3_check(mc->remote_server,
-					       mc->remote_username,
-					       mc->real_password);
+				mc->remote_handle =
+					helper_pop3_check (got_remote_answer,
+							   mc,
+							   null_remote_handle,
+							   mc->pre_remote_command,
+							   mc->remote_server,
+							   mc->remote_username,
+							   mc->real_password);
 			else
-				v = imap_check(mc->remote_server,
-					       mc->remote_username,
-					       mc->real_password);
+					helper_imap_check (got_remote_answer,
+							   mc,
+							   null_remote_handle,
+							   mc->pre_remote_command,
+							   mc->remote_server,
+							   mc->remote_username,
+							   mc->real_password);
 		}
-		else
-			v = -1;
-
-		if (v == -1) {
-#if 0
-			/* don't notify about an error: think of people with
-			 * dial-up connections; keep the current mail status
-			 */
-			GtkWidget *box = NULL;
-			box = gnome_message_box_new (_("Remote-client-error occured. Remote-polling deactivated. Maybe you used a wrong server/username/password?"),
-						     GNOME_MESSAGE_BOX_ERROR, GNOME_STOCK_BUTTON_CLOSE, NULL);
-			gtk_window_set_modal (GTK_WINDOW(box),TRUE);
-			gtk_widget_show (box);
-			
-			mc->mailbox_type = MAILBOX_LOCAL;
-			mc->anymail = mc->newmail = 0;
-#endif
-		}
-		else {
-			old_unreadmail = mc->unreadmail;
-			mc->unreadmail = (signed int) (((unsigned int) v) >> 16);
-			if(mc->unreadmail > old_unreadmail) /* lt */
-				mc->newmail = 1;
-			else
-				mc->newmail = 0;
-			mc->totalmail = (signed int) (((unsigned int) v) & 0x0000FFFFL);
-			mc->anymail = mc->totalmail ? 1 : 0;
-		} 
 	}
 	else if (mc->mailbox_type == MAILBOX_LOCAL) {
 		status = stat (mc->mail_file, &s);
@@ -373,34 +411,14 @@ next_frame (gpointer data)
 	return TRUE;
 }
 
-static int
-mail_check_timeout (gpointer data)
+static void
+after_mail_check (MailCheck *mc)
 {
 	static const char *supinfo[] = {"mailcheck", "new-mail", NULL};
-	MailCheck *mc = data;
-
-	if (mc->pre_check_enabled &&
-	    mc->pre_check_cmd && 
-	    (strlen(mc->pre_check_cmd) > 0)){
-		/*
-		 * if we have to execute a command before checking for mail, we
-		 * remove the mail-check timeout and re-add it after the command
-		 * returns, just in case the execution takes too long.
-		 */
-		
-		if(mc->mail_timeout != 0)
-			gtk_timeout_remove (mc->mail_timeout);
-		mc->mail_timeout = 0;
-		if (system(mc->pre_check_cmd) == 127)
-			g_warning("Couldn't execute command");
-		mc->mail_timeout = gtk_timeout_add(mc->update_freq, mail_check_timeout, mc);
-	}
-
-	check_mail_file_status (mc);
 
 	if(mc->newmail) {
-    if(mc->play_sound)
-      gnome_triggers_vdo("", "program", supinfo);
+		if(mc->play_sound)
+			gnome_triggers_vdo("", "program", supinfo);
 
 		if (mc->newmail_enabled && 
 		    mc->newmail_cmd && 
@@ -468,7 +486,35 @@ mail_check_timeout (gpointer data)
 		break;
 	}
 	}
-	return 1;
+}
+
+static gboolean
+mail_check_timeout (gpointer data)
+{
+	MailCheck *mc = data;
+
+	if (mc->pre_check_enabled &&
+	    mc->pre_check_cmd && 
+	    (strlen(mc->pre_check_cmd) > 0)){
+		/*
+		 * if we have to execute a command before checking for mail, we
+		 * remove the mail-check timeout and re-add it after the command
+		 * returns, just in case the execution takes too long.
+		 */
+		
+		if(mc->mail_timeout != 0)
+			gtk_timeout_remove (mc->mail_timeout);
+		mc->mail_timeout = 0;
+		if (system(mc->pre_check_cmd) == 127)
+			g_warning("Couldn't execute command");
+		mc->mail_timeout = gtk_timeout_add(mc->update_freq, mail_check_timeout, mc);
+	}
+
+	check_mail_file_status (mc);
+
+	after_mail_check (mc);
+
+	return TRUE;
 }
 
 /*
@@ -515,6 +561,7 @@ mailcheck_destroy (GtkWidget *widget, gpointer data)
 	g_free (mc->clicked_cmd);
 
 	g_free (mc->remote_server);
+	g_free (mc->pre_remote_command);
 	g_free (mc->remote_username);
 	g_free (mc->remote_password);
 	g_free (mc->real_password);
@@ -533,6 +580,9 @@ mailcheck_destroy (GtkWidget *widget, gpointer data)
 	if (mc->animation_tag != 0)
 		gtk_timeout_remove (mc->animation_tag);
 
+	if (mc->remote_handle != NULL)
+		helper_whack_handle (mc->remote_handle);
+
 	/* just for sanity */
 	memset(mc, 0, sizeof(MailCheck));
 
@@ -831,10 +881,27 @@ apply_properties_callback (GtkWidget *widget, gint page, gpointer data)
 
 	if (strlen(text) > 0)
 		mc->remote_password = g_strdup(text);
+
+	if (mc->pre_remote_command) {
+		g_free(mc->pre_remote_command);
+		mc->pre_remote_command = NULL;
+	}
+
+        text = gtk_entry_get_text (GTK_ENTRY(mc->pre_remote_command_entry));
+	
+	if (strlen(text) > 0)
+		mc->pre_remote_command = g_strdup(text);
         
 	mc->mailbox_type = mc->mailbox_type_temp;
 
 	mc->play_sound = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(mc->play_sound_check));
+
+	if ((mc->mailbox_type != MAILBOX_POP3) &&
+	    (mc->mailbox_type != MAILBOX_IMAP) &&
+	    (mc->remote_handle != NULL)) {
+		helper_whack_handle (mc->remote_handle);
+		mc->remote_handle = NULL;
+	}
 }
 
 static void
@@ -852,6 +919,8 @@ make_remote_widgets_sensitive(MailCheck *mc)
 	gtk_widget_set_sensitive (mc->remote_server_label, b);
 	gtk_widget_set_sensitive (mc->remote_password_label, b);
 	gtk_widget_set_sensitive (mc->remote_username_label, b);
+	gtk_widget_set_sensitive (mc->pre_remote_command_entry, b);
+	gtk_widget_set_sensitive (mc->pre_remote_command_label, b);
 }
 
 static void 
@@ -991,6 +1060,23 @@ mailbox_properties_page(MailCheck *mc)
 	
 	gtk_signal_connect(GTK_OBJECT(l), "changed",
                      GTK_SIGNAL_FUNC(property_box_changed), mc);
+
+	hbox = gtk_hbox_new (FALSE, 6);
+	gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
+	gtk_widget_show (hbox);  
+
+	mc->pre_remote_command_label = l = gtk_label_new(_("Command to run before we check for mail:"));
+	gtk_widget_show(l);
+	gtk_box_pack_start (GTK_BOX (hbox), l, FALSE, FALSE, 0);
+  
+	mc->pre_remote_command_entry = l = gtk_entry_new();
+	if (mc->pre_remote_command)
+		gtk_entry_set_text(GTK_ENTRY(l), mc->pre_remote_command);
+  	gtk_widget_show(l);
+	gtk_box_pack_start (GTK_BOX (hbox), l, TRUE, TRUE, 0);      
+	
+	gtk_signal_connect(GTK_OBJECT(l), "changed",
+			   GTK_SIGNAL_FUNC(property_box_changed), mc);
   
 	make_remote_widgets_sensitive(mc);
 	
@@ -1220,6 +1306,8 @@ applet_save_session(GtkWidget *w,
 				mc->remote_username?mc->remote_username:"");
         gnome_config_private_set_string("mail/remote_password", 
 				mc->remote_password?mc->remote_password:"");
+        gnome_config_private_set_string("mail/pre_remote_command", 
+				mc->pre_remote_command ? mc->pre_remote_command : "");
         gnome_config_set_int("mail/mailbox_type", (int) mc->mailbox_type);
 	gnome_config_set_bool("mail/play_sound", mc->play_sound);
 
@@ -1355,6 +1443,7 @@ make_mailcheck_applet(const gchar *goad_id)
 	mc->clicked_enabled = gnome_config_get_bool("mail/clicked_enabled=0");
 
 	mc->remote_server = gnome_config_private_get_string("mail/remote_server=mail");
+	mc->pre_remote_command = gnome_config_private_get_string("mail/pre_remote_command=");
 	
 	query = g_strconcat("mail/remote_username=", g_getenv("USER"), NULL);
 	mc->remote_username = gnome_config_private_get_string(query);
