@@ -20,6 +20,7 @@
 /* define to x for debugging output */
 #define d(x)
 
+#define APPLET_COMPILE_AS_PROCESS
 /* sorting the list... */
 static gint
 tlsort (gconstpointer data1, gconstpointer data2)
@@ -33,14 +34,25 @@ tlsort (gconstpointer data1, gconstpointer data2)
 	sa = la->task_group ? la->group_name : (la->group ? la->group->group_name : la->gwmh_task->name);
 	sb = lb->task_group ? lb->group_name : (lb->group ? lb->group->group_name : lb->gwmh_task->name);
 
-	if (sa && sb)
-		return g_strcasecmp (sa, sb);
-	else if (sa)
+	if (sa && sb) {
+		int cmp = g_strcasecmp (sa, sb);
+		/* if the same, then sort by the window id, this is arbitrary
+		 * but makes tasks of the same group stay in the same order */
+		if (cmp == 0) {
+			if (la->gwmh_task->xwin > lb->gwmh_task->xwin)
+				return 1;
+			else
+				return -1;
+		} else {
+			return cmp;
+		}
+	} else if (sa) {
 		return 1;
-	else if (sb)
+	} else if (sb) {
 		return -1;
-	else
+	} else {
 		return 0;
+	}
 }
 
 static void
@@ -112,6 +124,9 @@ tasklist_task_get_label (TasklistTask *task, int width, gboolean add_groupcount)
 
 	das_string = task->gwmh_task->name;
 
+	if(!das_string)
+	  return g_strdup("");
+
 	label_len = gdk_string_width (tasklist->area->style->font, das_string);
 
 	overhead = tasklist->config.show_mini_icons ? 30 : 6;
@@ -282,16 +297,19 @@ print_task (TasklistTask *task)
 	}
 }
 
-static void
+/* returns TRUE if the group entry should be redrawn */
+static gboolean
 fixup_group (TasklistTask *group)
 {
+	TasklistTask *focused_task;
+	gboolean iconified;
 	TasklistTask *task;
 	GSList *item;
 	
-	g_return_if_fail (group != NULL);
+	g_return_val_if_fail (group != NULL, FALSE);
 	
-	group->focused_task = NULL;
-	group->gwmh_task->iconified = TRUE;
+	focused_task = NULL;
+	iconified = TRUE;
 	
 	g_slist_free (group->vtasks);
 	group->vtasks = NULL;
@@ -301,32 +319,45 @@ fixup_group (TasklistTask *group)
 		if (is_task_visible (task)) {
 			group->vtasks = g_slist_prepend (group->vtasks, task);
 			if (!task->gwmh_task->iconified)
-				group->gwmh_task->iconified = FALSE;
+				iconified = FALSE;
 			if (task->gwmh_task->focused)
-				group->focused_task = task;
+				focused_task = task;
 		}
 	}
+
+	if (group->focused_task != focused_task ||
+	    group->gwmh_task->iconified != iconified) {
+		group->focused_task = focused_task;
+		group->gwmh_task->iconified = iconified;
+
+		return TRUE;
+	} else {
+		return FALSE;
+	}
 }
 
-static gboolean fixup_vtask (TasklistTask *task);
+static gboolean fixup_vtask (TasklistTask *task, gboolean *redraw);
 
 static void
 fixup_vtask_iterator (gpointer data, gpointer user_data)
 {
 	TasklistTask *task = data;
-	fixup_vtask (task);
+	fixup_vtask (task, NULL);
 }
 
 static gboolean
-fixup_vtask (TasklistTask *task)
+fixup_vtask (TasklistTask *task, gboolean *redraw)
 {
 	gboolean visible;
 
 	/* why not layout if we are confused */
 	g_return_val_if_fail (task, TRUE);
 
-	if (task->task_group)
-		fixup_group (task);
+	if (task->task_group) {
+		gboolean do_redraw = fixup_group (task);
+		if (redraw != NULL)
+			*redraw = do_redraw;
+	}
 
 	visible = is_task_really_visible (task);
 
@@ -346,7 +377,7 @@ fixup_vtask (TasklistTask *task)
 		if (task->task_group) {
 			g_slist_foreach (task->tasks, fixup_vtask_iterator, NULL);
 		} else if (task->group) {
-			fixup_vtask (task->group);
+			fixup_vtask (task->group, NULL);
 		}
 	}
 
@@ -568,8 +599,7 @@ max_width (GSList *tasks)
 
 		if (task->fullwidth < 0) {
 			s = tasklist_task_get_label (task, -1, task->task_group);
-			task->fullwidth = gdk_string_width (
-				task->tasklist->area->style->font, s);
+			task->fullwidth = gdk_string_width (task->tasklist->area->style->font, s);
 			g_free (s);
 		}
 
@@ -582,21 +612,19 @@ max_width (GSList *tasks)
 }
 
 /* Layout the tasklist */
-static int
-real_layout_tasklist (Tasklist *tasklist, gboolean call_change_size)
+static gboolean
+real_layout_tasklist (gpointer data)
 {
+	Tasklist *tasklist = data;
 	gint j = 0, k = 0, num = 0, p = 0;
 	TasklistTask *task;
 	GSList *temp = NULL;
-	/* gint extra_space; */
 	gint num_rows = 0, num_cols = 0;
 	gint curx = 0, cury = 0, curwidth = 0, curheight = 0;
 
-	tasklist->layout_timeout = 0;
+	tasklist->layout_idle = 0;
 	d(g_message ("Layout!"));
 
-	/*gdk_beep ();*/
-	
 	if (!tasklist->vtasks) {
 		d(g_message ("no tasks :(\n"));
 		gtk_widget_draw (tasklist->area, NULL);
@@ -613,9 +641,8 @@ real_layout_tasklist (Tasklist *tasklist, gboolean call_change_size)
 				tasklist->horz_width = tasklist->config.horz_width;
 			else
 				tasklist->horz_width = 4;
-			
-			if (call_change_size)
-				tasklist_change_size (tasklist, FALSE, -1);
+
+			tasklist_change_size (tasklist, FALSE, -1);
 			
 			gtk_widget_draw (tasklist->area, NULL);
 			return FALSE;
@@ -702,8 +729,7 @@ real_layout_tasklist (Tasklist *tasklist, gboolean call_change_size)
 		else
 			tasklist->horz_width = num_cols * curwidth + 4;
 
-		if (call_change_size)
-			tasklist_change_size (tasklist, FALSE, -1);
+		tasklist_change_size (tasklist, FALSE, -1);
 
 		break;
 
@@ -715,9 +741,8 @@ real_layout_tasklist (Tasklist *tasklist, gboolean call_change_size)
 				tasklist->vert_height = tasklist->config.vert_height;
 			else
 				tasklist->vert_height = 4;
-			
-			if (call_change_size)
-				tasklist_change_size (tasklist, FALSE, -1);
+
+			tasklist_change_size (tasklist, FALSE, -1);
 			
 			gtk_widget_draw (tasklist->area, NULL);
 			return FALSE;
@@ -746,10 +771,9 @@ real_layout_tasklist (Tasklist *tasklist, gboolean call_change_size)
 				clamp_size (tasklist, &tasklist->vert_height);
 			tasklist->vert_height -= DRAG_HANDLE_SIZE;
 		}
-		
-		if (call_change_size)
-			tasklist_change_size (tasklist, FALSE, curwidth);
 
+		tasklist_change_size (tasklist, FALSE, curwidth);
+		
 		for (temp = tasklist->vtasks; temp != NULL; temp = temp->next) {
 			task = (TasklistTask *) temp->data;
 			
@@ -783,13 +807,13 @@ tasklist_layout_tasklist (Tasklist *tasklist)
 	g_return_if_fail (tasklist);
 
 	/* don't queue another timeout */
-	if (tasklist->layout_timeout) {
+	if (tasklist->layout_idle) {
 		d(g_message ("Skipped layout!"));
 		return;
 	}
 	
 	d(g_message ("Adding layout callback..."));
-	tasklist->layout_timeout = g_idle_add ((GSourceFunc)real_layout_tasklist, tasklist);
+	tasklist->layout_idle = gtk_idle_add (real_layout_tasklist, tasklist);
 }
 
 #if 0
@@ -890,7 +914,7 @@ tasklist_task_destroy (GwmhTask *gtask, Tasklist *tasklist)
 		if (!ttask->group->tasks)
 			tasklist_group_destroy (ttask->group);
 		else
-			fixup_vtask (ttask->group);
+			fixup_vtask (ttask->group, NULL);
 	}
 
 	tasklist->vtasks = g_slist_remove (tasklist->vtasks, ttask);
@@ -971,7 +995,7 @@ tasklist_task_new (GwmhTask *gtask, Tasklist *tasklist)
 	else {
 		ttask->group->tasks = g_slist_prepend (ttask->group->tasks, ttask);
 		g_free (class);
-		fixup_vtask (ttask->group);
+		fixup_vtask (ttask->group, NULL);
 	}
 }
 
@@ -993,69 +1017,73 @@ task_notifier (gpointer func_data, GwmhTask *gwmh_task,
 {
 	Tasklist *tasklist = (Tasklist *) func_data;
 	TasklistTask *task;
-	gboolean resize = FALSE;
+	gboolean redraw = FALSE;
+	gboolean relayout = FALSE;
 
 	switch (ntype)
 	{
 	case GWMH_NOTIFY_INFO_CHANGED:
+		/* if this is just an allocation change, then no we don't want
+		 * to do anything */
+		if ( ! (imask & ~GWMH_TASK_INFO_ALLOCATION))
+			break;
+
 		task = g_hash_table_lookup (tasklist->tasks, gwmh_task);
 		if (!task) {
 			g_warning ("Getting info about task we don't know about: %p", gwmh_task);
 			break;
 		}
 
-		if (imask & GWMH_TASK_INFO_FOCUSED && task->gwmh_task->focused && task->group)
+		if ((imask & GWMH_TASK_INFO_FOCUSED) &&
+		    task->gwmh_task->focused && task->group)
 			task->group->focused_task = task;
 
-		/* we only need to re-layout if the task has changed
-		 * visibility status.  If it has, its group will also get fixed up
-		 */
 
-		/* this probably should be optimized and only done when the title changes */
-		resize = (tasklist->config.vert_width_full && 
-			  (tasklist->orient == ORIENT_LEFT || tasklist->orient == ORIENT_RIGHT));
-		task->fullwidth = -1;
-
-		if (fixup_vtask (task)) {
-			if (resize)
-				tasklist_change_size (tasklist, TRUE, -1);
-			else
-				tasklist_layout_tasklist (tasklist);
-		} else {
-			if (resize)
-				tasklist_change_size (tasklist, TRUE, -1);
-			else
-				tasklist_draw_task (task->group && is_task_really_visible (task->group)
-						    ? task->group : task, NULL);
+		if (imask & GWMH_TASK_INFO_MISC) {
+			if (tasklist->config.vert_width_full && 
+			    (tasklist->orient == ORIENT_LEFT ||
+			     tasklist->orient == ORIENT_RIGHT))
+				relayout = TRUE;
+			task->fullwidth = -1;
 		}
 
-#if 0
+		/* we only need to re-layout if the task has changed
+		 * visibility status.  If it has, its group will also get
+		 * fixed up
+		 */
+		if (fixup_vtask (task, &redraw)) {
+			relayout = TRUE;
+		}
+
+
 		if (imask & GWMH_TASK_INFO_WM_HINTS) {
-			if (tasklist_icon_get_pixmap (task) != task->wmhints_icon) {
-				tasklist_icon_destroy (tasklist, task);
-				tasklist_icon_set (tasklist, task);
-				tasklist_draw_task (tasklist, task, NULL);
+			if (tasklist_icon_get_pixmap (task) !=
+			    task->wmhints_icon) {
+				tasklist_icon_destroy (task);
+				tasklist_icon_set (task);
+				redraw = TRUE;
 			}
 		}
 		if (imask & GWMH_TASK_INFO_GSTATE)
-			tasklist_layout_tasklist (tasklist);
+			relayout = TRUE;
 		if (imask & GWMH_TASK_INFO_ICONIFIED)
-			tasklist_layout_tasklist (tasklist);
+			relayout = TRUE;
 
 		if (imask & GWMH_TASK_INFO_FOCUSED)
-			tasklist_draw_task (tasklist, task, NULL);
+			redraw = TRUE;
 		if (imask & GWMH_TASK_INFO_MISC)
-			tasklist_draw_task (tasklist, task, NULL);
-		if (imask & GWMH_TASK_INFO_DESKTOP) {
-			if (tasklist->config.all_desks_minimized && 
-			    tasklist->config.all_desks_normal)
-				break;
+			redraw = TRUE;
+		if (imask & GWMH_TASK_INFO_DESKTOP)
+			relayout = TRUE;
 
-		if (que_layout)
-			/* Redraw entire tasklist */
+		/* BTW, change_size is always called from the layout_tasklist
+		 * since we pass TRUE */
+		if (relayout)
 			tasklist_layout_tasklist (tasklist);
-		}
-#endif
+		else if (redraw)
+			tasklist_draw_task (task->group && is_task_really_visible (task->group)
+					    ? task->group : task, NULL);
+
 		break;
 	case GWMH_NOTIFY_NEW:
 		tasklist_task_new (gwmh_task, tasklist);
@@ -1234,7 +1262,6 @@ cb_properties (AppletWidget *applet, Tasklist *tasklist)
 static void
 cb_about (AppletWidget *applet, Tasklist *tasklist)
 {
-	static GtkWidget *dialog = NULL;
 
 	const char *authors[] = {
 		"Anders Carlsson (andersca@gnu.org)",
@@ -1245,27 +1272,27 @@ cb_about (AppletWidget *applet, Tasklist *tasklist)
 	};
 
 	/* Stop the about box from being shown twice at once */
-	if (dialog != NULL)
+	if (tasklist->about_dialog != NULL)
 	{
-		gtk_widget_show_now (dialog);
-		gdk_window_raise (dialog->window);
+		gtk_widget_show_now (tasklist->about_dialog);
+		gdk_window_raise (tasklist->about_dialog->window);
 		return;
 	}
 	
-	dialog = gnome_about_new (
+	tasklist->about_dialog = gnome_about_new (
 		"Gnome Tasklist",
 		VERSION,
 		_("Copyright (C) 1999 Anders Carlsson"),
 		authors,
 		_("A tasklist for the GNOME desktop environment.\nIcons by Tuomas Kuosmanen (tigert@gimp.org)."),
 		NULL);
-	gtk_signal_connect (GTK_OBJECT(dialog), "destroy",
-			    GTK_SIGNAL_FUNC(gtk_widget_destroyed), &dialog);
+	gtk_signal_connect (GTK_OBJECT(tasklist->about_dialog), "destroy",
+			    GTK_SIGNAL_FUNC(gtk_widget_destroyed), &tasklist->about_dialog);
 
-	gtk_widget_show_now (dialog);
+	gtk_widget_show_now (tasklist->about_dialog);
 
-	if (dialog->window != NULL)
-		gdk_window_raise (dialog->window);
+	if (tasklist->about_dialog->window != NULL)
+		gdk_window_raise (tasklist->about_dialog->window);
 }
 
 /* Ignore mouse button 1 clicks */
@@ -1370,6 +1397,58 @@ tasklist_destroy (GtkObject *applet_widget, Tasklist *tasklist)
 
 	tasklist->task_notifier_id = -1;
 	tasklist->desk_notifier_id = -1;
+
+	if (tasklist->layout_idle != 0) {
+		gtk_idle_remove (tasklist->layout_idle);
+		tasklist->layout_idle = 0;
+	}
+
+	if (tasklist->motion_timeout) {
+		gtk_timeout_remove (tasklist->motion_timeout);
+		tasklist->motion_timeout = 0;
+		tasklist->motion_task = NULL;
+	}
+
+	if (tasklist->prop)
+		gtk_widget_destroy (tasklist->prop);
+	tasklist->prop = NULL;
+
+	if (tasklist->about_dialog)
+		gtk_widget_destroy (tasklist->about_dialog);
+	tasklist->about_dialog = NULL;
+
+	if (tasklist->unknown_icon != NULL) {
+		if (tasklist->unknown_icon->normal != NULL) {
+			gdk_pixbuf_unref (tasklist->unknown_icon->normal);
+			tasklist->unknown_icon->normal = NULL;
+		}
+		if (tasklist->unknown_icon->minimized != NULL) {
+			gdk_pixbuf_unref (tasklist->unknown_icon->minimized);
+			tasklist->unknown_icon->minimized = NULL;
+		}
+		g_free (tasklist->unknown_icon);
+		tasklist->unknown_icon = NULL;
+	}
+
+	g_slist_free (tasklist->vtasks);
+	tasklist->vtasks = NULL;
+
+#ifndef APPLET_COMPILE_AS_PROCESS
+        #warning Here we save peoples memory by making this a shlib applet and leaking whatever we possibly can!
+#endif
+
+	if (tasklist->groups != NULL) {
+		/* FIXME: we leak EVERYTHING here */
+		g_hash_table_destroy (tasklist->groups);
+		tasklist->groups = NULL;
+	}
+	if (tasklist->tasks != NULL) {
+		/* FIXME: we leak EVERYTHING here */
+		g_hash_table_destroy (tasklist->tasks);
+		tasklist->tasks = NULL;
+	}
+
+	g_free (tasklist);
 }
 
 /* Create the applet */
@@ -1391,9 +1470,6 @@ tasklist_new (void)
 		return NULL;
 	}
 
-#warning should we be doing this?
-	/* gtk_widget_ref (tasklist->applet); */
-	
 	hbox = gtk_hbox_new (FALSE, 0);
 	gtk_widget_show (hbox);
 	
@@ -1524,11 +1600,9 @@ main (gint argc, gchar *argv[])
 
 	tasklist_init ();
 	
-	tasklist = create_applet ();
+	tasklist = tasklist_new ();
 
-	tasklist_change_size (tasklist, TRUE);
-
-	gtk_widget_show (applet);
+	gtk_widget_show_all (tasklist->applet);
 
 	applet_widget_gtk_main ();
 
