@@ -18,6 +18,7 @@ static gchar *contributors[] = {
 	"Shawn T. Amundson",
 	"Erik Andersen",
 	"Jon Anhold",
+	"Timur I. Bakeyev",
 	/* for languages that can't display aacute, replace '' with 'a' */
 	N_("Szabolcs 'Shooby' Bn"),
 	"Martin Baulig",
@@ -159,6 +160,8 @@ static gchar *contributors[] = {
 	"Gregory McLean",
 	"Michael Meeks",
 	"Federico Mena-Quintero",
+	"Christophe Merlet",
+	"Christian Meyer",
 	"Cesar Miquel",
 	"Julian Missig",
 	"Eric B. Mitchell",
