@@ -132,10 +132,23 @@ basep_widget_realize (GtkWidget *w)
 	g_return_if_fail (klass);
 	if (klass->realize != NULL)
 		klass->realize (w);
+}
 
-	/* make sure we reset our position, so that if the wm misplaces us
-	 * we jump back */
-	gtk_widget_queue_resize (w);
+static gboolean
+basep_widget_configure_event (GtkWidget *w, GdkEventConfigure *event)
+{
+	g_return_val_if_fail (IS_BASEP_WIDGET (w), FALSE);
+
+	if (event->x != w->allocation.x ||
+	    event->y != w->allocation.y ||
+	    event->width != w->allocation.width ||
+	    event->height != w->allocation.height)
+		gtk_widget_queue_resize (w);
+
+	if (GTK_WIDGET_CLASS (basep_widget_parent_class)->configure_event != NULL)
+		return GTK_WIDGET_CLASS (basep_widget_parent_class)->configure_event (w, event);
+	else
+		return FALSE;
 }
 
 static void
@@ -341,6 +354,7 @@ basep_widget_class_init (BasePWidgetClass *klass)
 	widget_class->size_request = basep_widget_size_request;
 	widget_class->size_allocate = basep_widget_size_allocate;
 	widget_class->realize = basep_widget_realize;
+	widget_class->configure_event = basep_widget_configure_event;
 	widget_class->enter_notify_event = basep_enter_notify;
 	widget_class->leave_notify_event = basep_leave_notify;
 	widget_class->style_set = basep_style_set;
@@ -1503,8 +1517,11 @@ basep_widget_set_hidebuttons (BasePWidget *basep)
 		gtk_widget_hide(basep->hidebutton_w);
 		gtk_widget_hide(basep->hidebutton_s);	
 
-		if (basep->state == BASEP_HIDDEN_LEFT ||
-		    basep->state == BASEP_HIDDEN_RIGHT)
+		/* if we removed hidebuttons we need to show ourselves,
+		 * except for the drawers case that is */
+		if ((basep->state == BASEP_HIDDEN_LEFT ||
+		     basep->state == BASEP_HIDDEN_RIGHT) &&
+		    ! IS_DRAWER_WIDGET (basep))
 			basep_widget_explicit_show (basep);
 	} else {
 		g_return_if_fail (klass && klass->set_hidebuttons);
@@ -1618,6 +1635,8 @@ basep_widget_explicit_show (BasePWidget *basep)
 void
 basep_widget_autoshow (BasePWidget *basep)
 {
+	static const char *supinfo[] = {"panel", "expand", NULL};
+
 	g_return_if_fail (IS_BASEP_WIDGET(basep));
 
 	if (basep->state == BASEP_MOVING) {
@@ -1636,6 +1655,8 @@ basep_widget_autoshow (BasePWidget *basep)
 		PanelOrientType hide_orient;
 		int w, h, size;
 
+		gnome_triggers_vdo("", NULL, supinfo);
+
 		hide_orient = klass->get_hide_orient (basep);
 		basep_widget_get_size (basep, &w, &h);
 		klass->get_hide_size (basep, 
@@ -1892,13 +1913,6 @@ basep_widget_get_pos (BasePWidget *basep,
 
 	basep_widget_get_size (basep, &w, &h);
 	klass->get_pos(basep, x, y, w, h);
-
-#if 0
-	/* sanity check, may not be desired */
-	*x = CLAMP (*x, 0, gdk_screen_width () - w);
-	*y = CLAMP (*y, 0, gdk_screen_height () - h);
-#endif
-
 }
 
 void
@@ -2049,6 +2063,13 @@ basep_calculate_borders (void)
 	}
 }
 
+static int
+border_max (BorderEdge edge)
+{
+	return MAX (borders[edge].center,
+		    MAX (borders[edge].left, borders[edge].right));
+}
+
 void
 basep_border_recalc (void)
 {
@@ -2083,6 +2104,12 @@ basep_border_recalc (void)
 		}
 	}
 
+	/* this does not generate xtraffic if not needed */
+	xstuff_setup_desktop_area (border_max (BORDER_LEFT),
+				   border_max (BORDER_RIGHT),
+				   border_max (BORDER_TOP) +
+				     foobar_widget_get_height (),
+				   border_max (BORDER_BOTTOM));
 }
 
 static guint queue_recalc_id = 0;
