@@ -20,6 +20,8 @@
 
 #include "panel-include.h"
 
+#include "icon-entry-hack.h"
+
 static void properties_apply (Launcher *launcher);
 
 extern GtkTooltips *panel_tooltips;
@@ -335,6 +337,7 @@ create_launcher (const char *parameters, GnomeDesktopEntry *dentry)
 
 	launcher = g_new0 (Launcher, 1);
 
+	launcher->info = NULL;
 	launcher->button = NULL;
 	launcher->dedit = NULL;
 	launcher->prop_dialog = NULL;
@@ -418,6 +421,7 @@ properties_apply (Launcher *launcher)
 {
 	char *icon;
 	char *location;
+	char *docpath;
 
 	/* save (steal) location */
 	location = launcher->dentry->location;
@@ -459,6 +463,22 @@ properties_apply (Launcher *launcher)
 		button_widget_set_pixmap(BUTTON_WIDGET(launcher->button),
 					 default_app_pixmap, -1);
 	}
+
+	applet_remove_callback (launcher->info, "help_on_app");
+	docpath = panel_gnome_kde_help_path (launcher->dentry->docpath);
+	if (docpath != NULL) {
+		char *title;
+
+		g_free (docpath);
+
+		title = g_strdup_printf (_("Help on %s"),
+					 launcher->dentry->name);
+
+		applet_add_callback (launcher->info, "help_on_app",
+				     GNOME_STOCK_PIXMAP_HELP,
+				     title);
+		g_free (title);
+	}
 }
 
 static void
@@ -525,6 +545,7 @@ create_properties_dialog (Launcher *launcher)
 	
 	launcher->dedit =
 		gnome_dentry_edit_new_notebook(GTK_NOTEBOOK(notebook));
+	hack_dentry_edit (GNOME_DENTRY_EDIT (launcher->dedit));
 	
 	types = NULL;
 	types = g_list_append (types, "Application");
@@ -605,6 +626,7 @@ load_launcher_applet_full (const char *params, GnomeDesktopEntry *dentry,
 			   PanelWidget *panel, int pos, gboolean exactpos)
 {
 	Launcher *launcher;
+	char *docpath;
 
 	launcher = create_launcher (params, dentry);
 
@@ -620,16 +642,34 @@ load_launcher_applet_full (const char *params, GnomeDesktopEntry *dentry,
 		return NULL;
 	}
 
+	launcher->info = applets_last->data;
+
 	gtk_tooltips_set_tip (panel_tooltips,
 			      launcher->button,
-			      launcher->dentry->comment,NULL);
+			      launcher->dentry->comment,
+			      NULL);
 
-	applet_add_callback(applets_last->data,"properties",
-			    GNOME_STOCK_MENU_PROP,
-			    _("Properties..."));
-	applet_add_callback(applets_last->data, "help",
-			    GNOME_STOCK_PIXMAP_HELP,
-			    _("Help"));
+	applet_add_callback (applets_last->data,"properties",
+			     GNOME_STOCK_MENU_PROP,
+			     _("Properties..."));
+	applet_add_callback (applets_last->data, "help",
+			     GNOME_STOCK_PIXMAP_HELP,
+			     _("Help"));
+
+	docpath = panel_gnome_kde_help_path (launcher->dentry->docpath);
+	if (docpath != NULL) {
+		char *title;
+
+		g_free (docpath);
+
+		title = g_strdup_printf (_("Help on %s"),
+					 launcher->dentry->name);
+
+		applet_add_callback (applets_last->data, "help_on_app",
+				     GNOME_STOCK_PIXMAP_HELP,
+				     title);
+		g_free (title);
+	}
 
 	return launcher;
 }
@@ -687,6 +727,7 @@ ask_about_launcher (const char *file, PanelWidget *panel, int pos, gboolean exac
 	gtk_box_pack_start (GTK_BOX(GNOME_DIALOG(dialog)->vbox), notebook,
 			    TRUE, TRUE, GNOME_PAD_SMALL);
 	dee = GNOME_DENTRY_EDIT(gnome_dentry_edit_new_notebook(GTK_NOTEBOOK(notebook)));
+	hack_dentry_edit (dee);
 
 	types = NULL;
 	types = g_list_append(types, "Application");
@@ -819,7 +860,11 @@ convert_dentry_to_gnome (GnomeDesktopEntry *dentry)
 
 	dentry->is_kde = FALSE;
 	for (i = 0; i < dentry->exec_length; i++) {
-		if (dentry->exec[i][0] == '%' &&
+		if (strcmp (dentry->exec[i], "\"%c\"") == 0 ||
+		    strcmp (dentry->exec[i], "%c") == 0) {
+			g_free (dentry->exec[i]);
+			dentry->exec[i] = g_strdup_printf ("'%s'", dentry->name);
+		} else if (dentry->exec[i][0] == '%' &&
 		    strlen(dentry->exec[i]) == 2) {
 			g_free (dentry->exec[i]);
 			dentry->exec[i] = g_strdup ("");
